# bash completion for the Aquilon client




_aq_sqlite="/ms/dist/fsf/PROJ/sqlite/3.8.2-1/bin/sqlite3"
_aq_datawarehouse="/ms/dist/aquilon/PROJ/datawarehouse/dumpv3"

_aq_complete_branch ()
{
    $_aq_sqlite $_aq_datawarehouse/hostdata.db "select distinct branch_name from hosts where branch_type = 'sandbox' and branch_name glob '$1*'"
}

_aq_complete_author_branch ()
{
    $_aq_sqlite $_aq_datawarehouse/hostdata.db "select distinct (branch_author || '/' || branch_name) from hosts where branch_type = 'sandbox'"
}

_aq_complete_hostname ()
{
    $_aq_sqlite $_aq_datawarehouse/hostdata.db "select hostname from hosts where hostname glob '$1*'"
}

_aq_complete_cluster ()
{
    $_aq_sqlite $_aq_datawarehouse/clusters.db "select name from cluster where name glob '$1*'"
}

_aq_complete_metacluster ()
{
    $_aq_sqlite $_aq_datawarehouse/clusters.db "select name from metacluster where name glob '$1*'"
}

_aq_complete_building ()
{
    $_aq_sqlite $_aq_datawarehouse/geography.db "select name from geography where loctype = 'building' and name glob '$1*'"
}

_aq_complete_region ()
{
    $_aq_sqlite $_aq_datawarehouse/geography.db "select name from geography where loctype = 'continent' and name glob '$1*'"
}

_aq_complete_city ()
{
    $_aq_sqlite $_aq_datawarehouse/geography.db "select name from geography where loctype = 'city' and name glob '$1*'"
}

_aq_complete_archetype ()
{
    $_aq_sqlite $_aq_datawarehouse/personality.db "select distinct archetype from personality where archetype glob '$1*'"
}

_aq_complete_personality ()
{
    $_aq_sqlite $_aq_datawarehouse/personality.db "select distinct name from personality where name glob '$1*'"
}

_aq_complete_service ()
{
    $_aq_sqlite $_aq_datawarehouse/servers.db "select distinct service from servers where service glob '$1*'"
}

_aq_complete_server_of_service () { _aq_complete_service $*; }

_aq_complete_instance ()
{
    $_aq_sqlite $_aq_datawarehouse/servers.db "select distinct instance from servers where instance glob '$1*'"
}

_aq_complete_server_of_instance () { _aq_complete_instance $*; }

_aq ()
{
    local cur cmds cmdOpts opt globalOpts globalParam optsParam optBase i

    COMPREPLY=()
    cur=${COMP_WORDS[COMP_CWORD]}

    cmds='status ping show_active_commands show_active_locks show_request flush show_host show_fqdn search_hardware search_machine search_system search_cluster search_metacluster search_host search_esx_cluster search_observed_mac search_next compile show_domain search_domain show_sandbox search_sandbox show_network search_network refresh_network refresh_windows_hosts add_cpu show_cpu del_cpu update_cpu add_disk del_disk update_disk add_host add_windows_host add_aurora_host add_auxiliary del_auxiliary add_manager show_manager del_manager add_model show_model search_model update_model del_model add_interface add_network_environment del_network_environment update_network_environment show_network_environment search_network_environment add_network_compartment show_network_compartment update_network_compartment del_network_compartment add_network del_network update_network split_network merge_network add_router_address del_router_address show_router_address update_router_address add_static_route del_static_route update_interface del_interface add_interface_address del_interface_address add_service show_service search_service update_service del_service bind_client rebind_client unbind_client bind_server unbind_server add_required_service show_archetype del_required_service map_service show_map unmap_service del_host del_windows_host pxeswitch reconfigure change_status reset_advertised_status add_domain update_archetype update_sandbox update_domain del_domain add_sandbox get publish deploy validate rollback del_sandbox manage cat sync add_chassis show_chassis del_chassis update_chassis add_rack update_rack show_rack del_rack add_desk show_desk del_desk add_building update_building show_building del_building add_campus del_campus update_campus add_room show_room del_room add_bunker show_bunker del_bunker show_campus add_city update_city show_city del_city add_country show_country del_country add_continent show_continent del_continent add_organization del_organization show_organization add_hub show_hub del_hub add_archetype del_archetype add_personality add_vendor del_vendor show_vendor update_vendor add_os update_os del_os show_os del_personality update_personality show_personality search_personality make make_cluster make_metacluster add_machine update_machine show_machine del_machine add_network_device add_switch update_network_device update_switch show_network_device show_switch search_network_device search_switch del_network_device del_switch poll_network_device poll_switch show_principal search_principal show_realm update_realm permission add_dns_environment show_dns_environment del_dns_environment update_dns_environment add_dns_domain show_dns_domain del_dns_domain update_dns_domain dump_dns add_metacluster show_metacluster update_metacluster del_metacluster show_cluster add_esx_cluster show_esx_cluster update_esx_cluster del_esx_cluster add_cluster update_cluster del_cluster add_allowed_personality del_allowed_personality rebind_metacluster bind_cluster rebind_cluster unbind_cluster cluster uncluster add_dynamic_range del_dynamic_range show_dynamic_range add_address del_address show_address update_address show_ns_record add_ns_record del_ns_record add_alias del_alias add_address_alias del_address_alias update_address_alias show_alias update_alias add_srv_record del_srv_record show_srv_record update_srv_record search_dns search_rack map_dns_domain unmap_dns_domain search_dns_domain_map search_audit show_reboot_schedule add_reboot_schedule del_reboot_schedule show_reboot_intervention add_reboot_intervention del_reboot_intervention add_filesystem del_filesystem show_filesystem add_share update_share del_share show_share add_application del_application show_application add_hostlink del_hostlink show_hostlink add_intervention show_intervention del_intervention add_service_address del_service_address show_service_address update_service_address add_grn del_grn update_grn show_grn refresh_grns map_grn unmap_grn add_resourcegroup del_resourcegroup show_resourcegroup update_resourcegroup add_feature update_feature del_feature show_feature bind_feature unbind_feature show_parameter validate_parameter add_parameter update_parameter del_parameter show_diff add_parameter_definition del_parameter_definition update_parameter_definition show_parameter_definition search_parameter_definition search_parameter add_vlan del_vlan show_vlan show_bunker_violations grant_root_access revoke_root_access add_netgroup_whitelist del_netgroup_whitelist show_netgroup_whitelist add_user del_user update_user show_user refresh_user add_virtual_switch del_virtual_switch show_virtual_switch update_virtual_switch bind_port_group unbind_port_group promote'
    globalOpts='--verbose --quiet --partialok --debug --httpinfo --format --auth --exec --usesock --slowstatus --aqport --aqhost --aquser --aqservice --aqconf'
    globalParam='--verbose|--quiet|--format|--usesock|--aqport|--aqhost|--aquser|--aqservice|--aqconf'

    if [[ $COMP_CWORD -eq 1 ]]; then
        COMPREPLY=( $( compgen -W "$cmds" -- $cur ) )
        return 0
    fi

    if [[ $COMP_CWORD -eq 2 && ${COMP_WORDS[1]} == "help" ]]; then
        COMPREPLY=( $( compgen -W "$cmds" -- $cur ) )
        return 0
    fi

    # auto-complete some options
    if [[ $COMP_CWORD -gt 2 && ! ( $cur =~ "^-") ]]; then
        local opt full_opt subject list

	full_opt=${COMP_WORDS[ $(( $COMP_CWORD - 1 )) ]}
	opt=${full_opt/--/}

	if [[ $opt == "name" ]]; then
	    # --name is a special case, extract from the command name
	    # what we're looking for
	    opt=${COMP_WORDS[1]##*_}
	fi

	if [[ $opt == "command" ]]; then
	    list=$cmds
	    COMPREPLY=( $( compgen -W "$list" -- $cur ) )
	    return 0
	fi

	if [[ $opt == "sandbox" ]]; then
            if [[ ${COMP_WORDS[1]} == "show_sandbox" ||
                  ${COMP_WORDS[1]} == "update_sandbox" ||
                  ${COMP_WORDS[1]} == "add_sandbox" ||
                  ${COMP_WORDS[1]} == "del_sandbox" ||
                  ${COMP_WORDS[1]} == "get"
                ]]; then

                # echo running _aq_complete_branch
                list=$( _aq_complete_branch $cur)
            else
                # echo running _aq_complete_author_branch
                list=$( _aq_complete_author_branch $cur)
            fi
            COMPREPLY=( $( compgen -W "$list" -- $cur ) )
            return 0
	fi

	subject=""
	if typeset -F _aq_complete_${opt} >/dev/null 2>&1; then
	    subject=${opt}
	fi

	if [[ -n "$subject" ]]; then
	    list=$( _aq_complete_${opt} $cur)
	    COMPREPLY=( $( compgen -W "$list" -- $cur ) )
	    return 0
	fi
    fi

    cmdOpts=
    case ${COMP_WORDS[1]} in

    status)
        cmdOpts=''
        optsParam=''
        ;;

    ping)
        cmdOpts=''
        optsParam=''
        ;;

    show_active_commands)
        cmdOpts=''
        optsParam=''
        ;;

    show_active_locks)
        cmdOpts=''
        optsParam=''
        ;;

    show_request)
        cmdOpts='--requestid --auditid'
        optsParam='--requestid|--auditid'
        ;;

    flush)
        cmdOpts='--hosts --machines --personalities --clusters --services --resources --locations --networks --network_devices --virtual_switches --all --justification --reason'
        optsParam='--hosts|--machines|--personalities|--clusters|--services|--resources|--locations|--networks|--network_devices|--virtual_switches|--all|--justification|--reason'
        ;;

    show_host)
        cmdOpts='--hostname --list --all --grns'
        optsParam='--hostname|--list|--all|--grns'
        ;;

    show_fqdn)
        cmdOpts='--fqdn --all --dns_environment'
        optsParam='--fqdn|--all|--dns_environment'
        ;;

    search_hardware)
        cmdOpts='--model --machine_type --vendor --ip --networkip --network_environment --mac --pg --interface_name --interface_model --interface_vendor --interface_bus_address --serial --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --exact_location --all --fullinfo'
        optsParam='--model|--machine_type|--vendor|--ip|--networkip|--network_environment|--mac|--pg|--interface_name|--interface_model|--interface_vendor|--interface_bus_address|--serial|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--exact_location|--all|--fullinfo'
        ;;

    search_machine)
        cmdOpts='--hostname --machine --cpuname --cpuvendor --cpucount --memory --cluster --metacluster --vmhost --model --machine_type --vendor --ip --networkip --network_environment --mac --interface_name --interface_model --interface_vendor --interface_bus_address --pg --serial --uuid --disk_name --disk_size --disk_controller --disk_address --disk_bus_address --disk_wwn --disk_share --disk_filesystem --share --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --exact_location --chassis --slot --used --fullinfo'
        optsParam='--hostname|--machine|--cpuname|--cpuvendor|--cpucount|--memory|--cluster|--metacluster|--vmhost|--model|--machine_type|--vendor|--ip|--networkip|--network_environment|--mac|--interface_name|--interface_model|--interface_vendor|--interface_bus_address|--pg|--serial|--uuid|--disk_name|--disk_size|--disk_controller|--disk_address|--disk_bus_address|--disk_wwn|--disk_share|--disk_filesystem|--share|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--exact_location|--chassis|--slot|--fullinfo'
        ;;

    search_system)
        cmdOpts='--fqdn --dns_domain --shortname --type --ip --networkip --mac --fullinfo'
        optsParam='--fqdn|--dns_domain|--shortname|--type|--ip|--networkip|--mac|--fullinfo'
        ;;

    search_cluster)
        cmdOpts='--archetype --buildstatus --cluster_type --personality --personality_stage --allowed_archetype --allowed_personality --down_hosts_threshold --down_maint_threshold --max_members --grouped_with --member_archetype --member_hostname --member_personality --domain --sandbox --branch --sandbox_author --cluster --metacluster --esx_metacluster --esx_virtual_machine --esx_guest --service --instance --esx_share --share --esx_switch --cluster_organization --cluster_hub --cluster_continent --cluster_country --cluster_campus --cluster_city --cluster_building --cluster_room --cluster_bunker --cluster_rack --cluster_desk --cluster_exact_location --member_organization --member_hub --member_continent --member_country --member_campus --member_city --member_building --member_room --member_bunker --member_rack --member_desk --member_exact_location --fullinfo'
        optsParam='--archetype|--buildstatus|--cluster_type|--personality|--personality_stage|--allowed_archetype|--allowed_personality|--down_hosts_threshold|--down_maint_threshold|--max_members|--grouped_with|--member_archetype|--member_hostname|--member_personality|--domain|--sandbox|--branch|--sandbox_author|--cluster|--metacluster|--esx_metacluster|--esx_virtual_machine|--esx_guest|--service|--instance|--esx_share|--share|--esx_switch|--cluster_organization|--cluster_hub|--cluster_continent|--cluster_country|--cluster_campus|--cluster_city|--cluster_building|--cluster_room|--cluster_bunker|--cluster_rack|--cluster_desk|--cluster_exact_location|--member_organization|--member_hub|--member_continent|--member_country|--member_campus|--member_city|--member_building|--member_room|--member_bunker|--member_rack|--member_desk|--member_exact_location|--fullinfo'
        ;;

    search_metacluster)
        cmdOpts='--archetype --personality --personality_stage --buildstatus --allowed_archetype --allowed_personality --max_members --member_archetype --member_cluster --member_personality --domain --sandbox --branch --sandbox_author --metacluster --service --instance --share --metacluster_organization --metacluster_hub --metacluster_continent --metacluster_country --metacluster_campus --metacluster_city --metacluster_building --metacluster_room --metacluster_bunker --metacluster_rack --metacluster_desk --metacluster_exact_location --member_organization --member_hub --member_continent --member_country --member_campus --member_city --member_building --member_room --member_bunker --member_rack --member_desk --member_exact_location --fullinfo'
        optsParam='--archetype|--personality|--personality_stage|--buildstatus|--allowed_archetype|--allowed_personality|--max_members|--member_archetype|--member_cluster|--member_personality|--domain|--sandbox|--branch|--sandbox_author|--metacluster|--service|--instance|--share|--metacluster_organization|--metacluster_hub|--metacluster_continent|--metacluster_country|--metacluster_campus|--metacluster_city|--metacluster_building|--metacluster_room|--metacluster_bunker|--metacluster_rack|--metacluster_desk|--metacluster_exact_location|--member_organization|--member_hub|--member_continent|--member_country|--member_campus|--member_city|--member_building|--member_room|--member_bunker|--member_rack|--member_desk|--member_exact_location|--fullinfo'
        ;;

    search_host)
        cmdOpts='--hostname --machine --dns_domain --shortname --archetype --buildstatus --ip --networkip --network_environment --mac --personality --personality_stage --host_environment --osname --osversion --service --instance --model --machine_type --vendor --serial --cluster --metacluster --guest_on_cluster --guest_on_share --member_cluster_share --domain --sandbox --branch --sandbox_author --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --server_of_service --server_of_instance --exact_location --grn --eon_id --fullinfo'
        optsParam='--hostname|--machine|--dns_domain|--shortname|--archetype|--buildstatus|--ip|--networkip|--network_environment|--mac|--personality|--personality_stage|--host_environment|--osname|--osversion|--service|--instance|--model|--machine_type|--vendor|--serial|--cluster|--metacluster|--guest_on_cluster|--guest_on_share|--member_cluster_share|--domain|--sandbox|--branch|--sandbox_author|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--server_of_service|--server_of_instance|--exact_location|--grn|--eon_id|--fullinfo'
        ;;

    search_esx_cluster)
        cmdOpts='--cluster --metacluster --esx_hostname --virtual_machine --guest --archetype --personality --personality_stage --service --instance --share --switch --buildstatus --domain --sandbox --branch --cluster_organization --cluster_hub --cluster_continent --cluster_country --cluster_campus --cluster_city --cluster_building --cluster_room --cluster_bunker --cluster_rack --cluster_desk --cluster_exact_location --vmhost_organization --vmhost_hub --vmhost_continent --vmhost_country --vmhost_campus --vmhost_city --vmhost_building --vmhost_room --vmhost_bunker --vmhost_rack --vmhost_desk --vmhost_exact_location --fullinfo'
        optsParam='--cluster|--metacluster|--esx_hostname|--virtual_machine|--guest|--archetype|--personality|--personality_stage|--service|--instance|--share|--switch|--buildstatus|--domain|--sandbox|--branch|--cluster_organization|--cluster_hub|--cluster_continent|--cluster_country|--cluster_campus|--cluster_city|--cluster_building|--cluster_room|--cluster_bunker|--cluster_rack|--cluster_desk|--cluster_exact_location|--vmhost_organization|--vmhost_hub|--vmhost_continent|--vmhost_country|--vmhost_campus|--vmhost_city|--vmhost_building|--vmhost_room|--vmhost_bunker|--vmhost_rack|--vmhost_desk|--vmhost_exact_location|--fullinfo'
        ;;

    search_observed_mac)
        cmdOpts='--network_device --switch --port --mac'
        optsParam='--network_device|--switch|--port|--mac'
        ;;

    search_next)
        cmdOpts='--machine --short --dns_domain --cluster --metacluster --number --pack --start'
        optsParam='--machine|--short|--dns_domain|--cluster|--metacluster|--number|--pack|--start'
        ;;

    compile)
        cmdOpts='--domain --sandbox --hostname --cluster --metacluster --personality --archetype --personality_stage --pancinclude --pancexclude --pancdebug --cleandeps'
        optsParam='--domain|--sandbox|--hostname|--cluster|--metacluster|--personality|--archetype|--personality_stage|--pancinclude|--pancexclude'
        ;;

    show_domain)
        cmdOpts='--domain --all'
        optsParam='--domain|--all'
        ;;

    search_domain)
        cmdOpts='--track --not_tracking --change_manager --owner --compiler_version --autosync --validated --used --compileable --fullinfo'
        optsParam='--track|--not_tracking|--owner|--compiler_version|--compileable|--fullinfo'
        ;;

    show_sandbox)
        cmdOpts='--sandbox --all --pathonly'
        optsParam='--sandbox|--all|--pathonly'
        ;;

    search_sandbox)
        cmdOpts='--owner --compiler_version --autosync --validated --used --compileable --fullinfo'
        optsParam='--owner|--compiler_version|--compileable|--fullinfo'
        ;;

    show_network)
        cmdOpts='--network --ip --all --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --exact_location --network_environment --hosts'
        optsParam='--network|--ip|--all|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--exact_location|--network_environment|--hosts'
        ;;

    search_network)
        cmdOpts='--network --network_environment --network_compartment --ip --type --side --machine --fqdn --cluster --pg --has_dynamic_ranges --exact_location --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --fullinfo'
        optsParam='--network|--network_environment|--network_compartment|--ip|--type|--side|--machine|--fqdn|--cluster|--pg|--has_dynamic_ranges|--exact_location|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--fullinfo'
        ;;

    refresh_network)
        cmdOpts='--building --all --dryrun --incremental --justification --reason'
        optsParam='--building|--all|--justification|--reason'
        ;;

    refresh_windows_hosts)
        cmdOpts='--dryrun --justification --reason'
        optsParam='--justification|--reason'
        ;;

    add_cpu)
        cmdOpts='--cpu --vendor --comments --justification --reason'
        optsParam='--cpu|--vendor|--comments|--justification|--reason'
        ;;

    show_cpu)
        cmdOpts='--all --cpu --vendor'
        optsParam='--all|--cpu|--vendor'
        ;;

    del_cpu)
        cmdOpts='--cpu --vendor --justification --reason'
        optsParam='--cpu|--vendor|--justification|--reason'
        ;;

    update_cpu)
        cmdOpts='--cpu --vendor --comments --justification --reason'
        optsParam='--cpu|--vendor|--comments|--justification|--reason'
        ;;

    add_disk)
        cmdOpts='--machine --disk --controller --size --share --filesystem --resourcegroup --address --comments --boot --snapshot --wwn --bus_address --iops_limit --justification --reason'
        optsParam='--machine|--disk|--controller|--size|--share|--filesystem|--resourcegroup|--address|--comments|--wwn|--bus_address|--iops_limit|--justification|--reason'
        ;;

    del_disk)
        cmdOpts='--machine --disk --all --justification --reason'
        optsParam='--machine|--disk|--all|--justification|--reason'
        ;;

    update_disk)
        cmdOpts='--machine --disk --rename_to --share --filesystem --resourcegroup --address --controller --size --comments --boot --snapshot --wwn --bus_address --iops_limit --justification --reason'
        optsParam='--machine|--disk|--rename_to|--share|--filesystem|--resourcegroup|--address|--controller|--size|--comments|--wwn|--bus_address|--iops_limit|--justification|--reason'
        ;;

    add_host)
        cmdOpts='--hostname --prefix --machine --archetype --domain --sandbox --ip --ipfromip --ipfromsystem --autoip --osname --osversion --buildstatus --personality --personality_stage --ipalgorithm --zebra_interfaces --comments --dns_domain --justification --reason --grn --eon_id'
        optsParam='--hostname|--prefix|--machine|--archetype|--domain|--sandbox|--ip|--ipfromip|--ipfromsystem|--autoip|--osname|--osversion|--buildstatus|--personality|--personality_stage|--ipalgorithm|--zebra_interfaces|--comments|--dns_domain|--justification|--reason|--grn|--eon_id'
        ;;

    add_windows_host)
        cmdOpts='--hostname --machine --buildstatus --personality --osversion --comments --ipalgorithm --justification --reason --ip --ipfromip --ipfromsystem --autoip --grn --eon_id'
        optsParam='--hostname|--machine|--buildstatus|--personality|--osversion|--comments|--ipalgorithm|--justification|--reason|--ip|--ipfromip|--ipfromsystem|--autoip|--grn|--eon_id'
        ;;

    add_aurora_host)
        cmdOpts='--hostname --osname --osversion --comments --justification --reason'
        optsParam='--hostname|--osname|--osversion|--comments|--justification|--reason'
        ;;

    add_auxiliary)
        cmdOpts='--hostname --machine --auxiliary --ip --ipfromip --ipfromsystem --autoip --interface --mac --ipalgorithm --comments --justification --reason'
        optsParam='--hostname|--machine|--auxiliary|--ip|--ipfromip|--ipfromsystem|--autoip|--interface|--mac|--ipalgorithm|--comments|--justification|--reason'
        ;;

    del_auxiliary)
        cmdOpts='--auxiliary --justification --reason'
        optsParam='--auxiliary|--justification|--reason'
        ;;

    add_manager)
        cmdOpts='--hostname --ip --ipfromip --ipfromsystem --autoip --manager --interface --mac --ipalgorithm --comments --justification --reason'
        optsParam='--hostname|--ip|--ipfromip|--ipfromsystem|--autoip|--manager|--interface|--mac|--ipalgorithm|--comments|--justification|--reason'
        ;;

    show_manager)
        cmdOpts='--manager --all'
        optsParam='--manager|--all'
        ;;

    del_manager)
        cmdOpts='--manager --justification --reason'
        optsParam='--manager|--justification|--reason'
        ;;

    add_model)
        cmdOpts='--model --vendor --type --cpuname --cpunum --memory --disktype --diskcontroller --disksize --cpuvendor --nicmodel --nicvendor --comments --justification --reason'
        optsParam='--model|--vendor|--type|--cpuname|--cpunum|--memory|--disktype|--diskcontroller|--disksize|--cpuvendor|--nicmodel|--nicvendor|--comments|--justification|--reason'
        ;;

    show_model)
        cmdOpts='--model --all --vendor --type'
        optsParam='--model|--all|--vendor|--type'
        ;;

    search_model)
        cmdOpts='--model --vendor --machine_type --cpuname --cpuvendor --cpunum --nicmodel --nicvendor --memory --disktype --diskcontroller --disksize --fullinfo'
        optsParam='--model|--vendor|--machine_type|--cpuname|--cpuvendor|--cpunum|--nicmodel|--nicvendor|--memory|--disktype|--diskcontroller|--disksize|--fullinfo'
        ;;

    update_model)
        cmdOpts='--model --vendor --newmodel --newvendor --machine_type --cpuname --cpuvendor --cpunum --memory --disktype --diskcontroller --disksize --nicmodel --nicvendor --comments --update_existing_machines --justification --reason'
        optsParam='--model|--vendor|--newmodel|--newvendor|--machine_type|--cpuname|--cpuvendor|--cpunum|--memory|--disktype|--diskcontroller|--disksize|--nicmodel|--nicvendor|--comments|--update_existing_machines|--justification|--reason'
        ;;

    del_model)
        cmdOpts='--model --vendor --justification --reason'
        optsParam='--model|--vendor|--justification|--reason'
        ;;

    add_interface)
        cmdOpts='--machine --hostname --chassis --network_device --switch --interface --mac --automac --pg --autopg --iftype --type --comments --model --vendor --bus_address --justification --reason'
        optsParam='--machine|--hostname|--chassis|--network_device|--switch|--interface|--mac|--automac|--pg|--autopg|--iftype|--type|--comments|--model|--vendor|--bus_address|--justification|--reason'
        ;;

    add_network_environment)
        cmdOpts='--network_environment --dns_environment --building --comments --justification --reason'
        optsParam='--network_environment|--dns_environment|--building|--comments|--justification|--reason'
        ;;

    del_network_environment)
        cmdOpts='--network_environment --justification --reason'
        optsParam='--network_environment|--justification|--reason'
        ;;

    update_network_environment)
        cmdOpts='--network_environment --building --clear_location --comments --justification --reason'
        optsParam='--network_environment|--building|--clear_location|--comments|--justification|--reason'
        ;;

    show_network_environment)
        cmdOpts='--network_environment --all'
        optsParam='--network_environment|--all'
        ;;

    search_network_environment)
        cmdOpts='--network_environment --building --fullinfo'
        optsParam='--network_environment|--building|--fullinfo'
        ;;

    add_network_compartment)
        cmdOpts='--network_compartment --comments --justification --reason'
        optsParam='--network_compartment|--comments|--justification|--reason'
        ;;

    show_network_compartment)
        cmdOpts='--network_compartment --all'
        optsParam='--network_compartment|--all'
        ;;

    update_network_compartment)
        cmdOpts='--network_compartment --comments --justification --reason'
        optsParam='--network_compartment|--comments|--justification|--reason'
        ;;

    del_network_compartment)
        cmdOpts='--network_compartment --justification --reason'
        optsParam='--network_compartment|--justification|--reason'
        ;;

    add_network)
        cmdOpts='--network --ip --netmask --prefixlen --mask --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --network_environment --network_compartment --type --side --comments --justification --reason'
        optsParam='--network|--ip|--netmask|--prefixlen|--mask|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--network_environment|--network_compartment|--type|--side|--comments|--justification|--reason'
        ;;

    del_network)
        cmdOpts='--ip --network_environment --justification --reason'
        optsParam='--ip|--network_environment|--justification|--reason'
        ;;

    update_network)
        cmdOpts='--network --ip --type --side --comments --network_environment --network_compartment --justification --reason --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk'
        optsParam='--network|--ip|--type|--side|--comments|--network_environment|--network_compartment|--justification|--reason|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk'
        ;;

    split_network)
        cmdOpts='--ip --netmask --prefixlen --network_environment --justification --reason'
        optsParam='--ip|--netmask|--prefixlen|--network_environment|--justification|--reason'
        ;;

    merge_network)
        cmdOpts='--ip --netmask --prefixlen --network_environment --justification --reason'
        optsParam='--ip|--netmask|--prefixlen|--network_environment|--justification|--reason'
        ;;

    add_router_address)
        cmdOpts='--fqdn --network_environment --building --ip --comments --justification --reason'
        optsParam='--fqdn|--network_environment|--building|--ip|--comments|--justification|--reason'
        ;;

    del_router_address)
        cmdOpts='--fqdn --ip --network_environment --justification --reason'
        optsParam='--fqdn|--ip|--network_environment|--justification|--reason'
        ;;

    show_router_address)
        cmdOpts='--fqdn --ip --all --network_environment'
        optsParam='--fqdn|--ip|--all|--network_environment'
        ;;

    update_router_address)
        cmdOpts='--fqdn --ip --network_environment --dns_environment --building --comments --clear_location --justification --reason'
        optsParam='--fqdn|--ip|--network_environment|--dns_environment|--building|--comments|--clear_location|--justification|--reason'
        ;;

    add_static_route)
        cmdOpts='--ip --netmask --prefixlen --gateway --networkip --personality --personality_stage --archetype --network_environment --comments --justification --reason'
        optsParam='--ip|--netmask|--prefixlen|--gateway|--networkip|--personality|--personality_stage|--archetype|--network_environment|--comments|--justification|--reason'
        ;;

    del_static_route)
        cmdOpts='--gateway --ip --netmask --prefixlen --personality --personality_stage --archetype --network_environment --justification --reason'
        optsParam='--gateway|--ip|--netmask|--prefixlen|--personality|--personality_stage|--archetype|--network_environment|--justification|--reason'
        ;;

    update_interface)
        cmdOpts='--machine --hostname --network_device --switch --chassis --interface --pg --autopg --mac --boot --default_route --comments --master --clear_master --model --vendor --rename_to --bus_address --justification --reason'
        optsParam='--machine|--hostname|--network_device|--switch|--chassis|--interface|--pg|--autopg|--mac|--boot|--comments|--master|--clear_master|--model|--vendor|--rename_to|--bus_address|--justification|--reason'
        ;;

    del_interface)
        cmdOpts='--interface --machine --switch --network_device --chassis --mac --justification --reason'
        optsParam='--interface|--machine|--switch|--network_device|--chassis|--mac|--justification|--reason'
        ;;

    add_interface_address)
        cmdOpts='--machine --chassis --network_device --switch --interface --fqdn --network_environment --label --ipalgorithm --map_to_primary --shared --priority --justification --reason --ip --ipfromip --ipfromsystem --autoip'
        optsParam='--machine|--chassis|--network_device|--switch|--interface|--fqdn|--network_environment|--label|--ipalgorithm|--shared|--priority|--justification|--reason|--ip|--ipfromip|--ipfromsystem|--autoip'
        ;;

    del_interface_address)
        cmdOpts='--machine --chassis --network_device --switch --interface --fqdn --ip --label --keep_dns --network_environment --justification --reason'
        optsParam='--machine|--chassis|--network_device|--switch|--interface|--fqdn|--ip|--label|--keep_dns|--network_environment|--justification|--reason'
        ;;

    add_service)
        cmdOpts='--service --comments --instance --need_client_list --justification --reason'
        optsParam='--service|--comments|--instance|--justification|--reason'
        ;;

    show_service)
        cmdOpts='--service --server --client --all --instance'
        optsParam='--service|--server|--client|--all|--instance'
        ;;

    search_service)
        cmdOpts='--service --instance --server_hostname --server_cluster --server_metacluster --server_organization --server_hub --server_continent --server_country --server_campus --server_city --server_building --server_room --server_bunker --server_rack --server_desk --client_hostname --client_cluster --client_metacluster --client_organization --client_hub --client_continent --client_country --client_campus --client_city --client_building --client_room --client_bunker --client_rack --client_desk --has_clients --server_exact_location --client_exact_location --fullinfo'
        optsParam='--service|--instance|--server_hostname|--server_cluster|--server_metacluster|--server_organization|--server_hub|--server_continent|--server_country|--server_campus|--server_city|--server_building|--server_room|--server_bunker|--server_rack|--server_desk|--client_hostname|--client_cluster|--client_metacluster|--client_organization|--client_hub|--client_continent|--client_country|--client_campus|--client_city|--client_building|--client_room|--client_bunker|--client_rack|--client_desk|--server_exact_location|--client_exact_location|--fullinfo'
        ;;

    update_service)
        cmdOpts='--service --max_clients --default --need_client_list --instance --comments --justification --reason'
        optsParam='--service|--max_clients|--default|--instance|--comments|--justification|--reason'
        ;;

    del_service)
        cmdOpts='--service --instance --justification --reason'
        optsParam='--service|--instance|--justification|--reason'
        ;;

    bind_client)
        cmdOpts='--hostname --cluster --metacluster --service --instance --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--service|--instance|--justification|--reason'
        ;;

    rebind_client)
        cmdOpts='--hostname --cluster --metacluster --service --instance --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--service|--instance|--justification|--reason'
        ;;

    unbind_client)
        cmdOpts='--hostname --cluster --metacluster --service --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--service|--justification|--reason'
        ;;

    bind_server)
        cmdOpts='--service --instance --hostname --cluster --alias --ip --service_address --resourcegroup --position --justification --reason'
        optsParam='--service|--instance|--hostname|--cluster|--alias|--ip|--service_address|--resourcegroup|--position|--justification|--reason'
        ;;

    unbind_server)
        cmdOpts='--service --instance --all --position --hostname --cluster --alias --ip --service_address --resourcegroup --justification --reason'
        optsParam='--service|--instance|--all|--position|--hostname|--cluster|--alias|--ip|--service_address|--resourcegroup|--justification|--reason'
        ;;

    add_required_service)
        cmdOpts='--service --archetype --personality --osname --osversion --personality_stage --environment_override --justification --reason'
        optsParam='--service|--archetype|--personality|--osname|--osversion|--personality_stage|--environment_override|--justification|--reason'
        ;;

    show_archetype)
        cmdOpts='--archetype --all'
        optsParam='--archetype|--all'
        ;;

    del_required_service)
        cmdOpts='--service --archetype --personality --osname --osversion --personality_stage --justification --reason'
        optsParam='--service|--archetype|--personality|--osname|--osversion|--personality_stage|--justification|--reason'
        ;;

    map_service)
        cmdOpts='--service --instance --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --networkip --archetype --personality --host_environment --justification --reason'
        optsParam='--service|--instance|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--networkip|--archetype|--personality|--host_environment|--justification|--reason'
        ;;

    show_map)
        cmdOpts='--service --instance --archetype --personality --host_environment --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --networkip --all --include_parents'
        optsParam='--service|--instance|--archetype|--personality|--host_environment|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--networkip|--all|--include_parents'
        ;;

    unmap_service)
        cmdOpts='--service --instance --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --networkip --archetype --personality --host_environment --justification --reason'
        optsParam='--service|--instance|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--networkip|--archetype|--personality|--host_environment|--justification|--reason'
        ;;

    del_host)
        cmdOpts='--hostname --justification --reason'
        optsParam='--hostname|--justification|--reason'
        ;;

    del_windows_host)
        cmdOpts='--hostname --justification --reason'
        optsParam='--hostname|--justification|--reason'
        ;;

    pxeswitch)
        cmdOpts='--hostname --list --status --localboot --install --firmware --blindbuild --rescue --configure --justification --reason'
        optsParam='--hostname|--list|--status|--localboot|--install|--firmware|--blindbuild|--rescue|--justification|--reason'
        ;;

    reconfigure)
        cmdOpts='--hostname --hostlist --list --membersof --osname --osversion --archetype --personality --personality_stage --buildstatus --keepbindings --cleargrn --comments --justification --reason --grn --eon_id'
        optsParam='--hostname|--hostlist|--list|--membersof|--osname|--osversion|--archetype|--personality|--personality_stage|--buildstatus|--cleargrn|--comments|--justification|--reason|--grn|--eon_id'
        ;;

    change_status)
        cmdOpts='--hostname --cluster --metacluster --buildstatus --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--buildstatus|--justification|--reason'
        ;;

    reset_advertised_status)
        cmdOpts='--hostname --list --justification --reason'
        optsParam='--hostname|--list|--justification|--reason'
        ;;

    add_domain)
        cmdOpts='--domain --track --start --comments --change_manager --allow_manage --justification --reason'
        optsParam='--domain|--track|--start|--comments|--justification|--reason'
        ;;

    update_archetype)
        cmdOpts='--archetype --compilable --description --cluster_type --comments --justification --reason'
        optsParam='--archetype|--description|--cluster_type|--comments|--justification|--reason'
        ;;

    update_sandbox)
        cmdOpts='--sandbox --comments --compiler_version --autosync --profile_formats --justification --reason'
        optsParam='--sandbox|--comments|--compiler_version|--profile_formats|--justification|--reason'
        ;;

    update_domain)
        cmdOpts='--domain --comments --compiler_version --autosync --change_manager --allow_manage --profile_formats --archived --justification --reason'
        optsParam='--domain|--comments|--compiler_version|--profile_formats|--justification|--reason'
        ;;

    del_domain)
        cmdOpts='--domain --justification --reason'
        optsParam='--domain|--justification|--reason'
        ;;

    add_sandbox)
        cmdOpts='--sandbox --start --get --comments --justification --reason'
        optsParam='--sandbox|--start|--comments|--justification|--reason'
        ;;

    get)
        cmdOpts='--sandbox'
        optsParam='--sandbox'
        ;;

    publish)
        cmdOpts='--branch --sandbox --sync --rebase --justification --reason'
        optsParam='--branch|--sandbox|--rebase|--justification|--reason'
        ;;

    deploy)
        cmdOpts='--source --target --sync --dryrun --justification --reason'
        optsParam='--source|--target|--justification|--reason'
        ;;

    validate)
        cmdOpts='--branch --comments'
        optsParam='--branch|--comments'
        ;;

    rollback)
        cmdOpts='--domain --ref --lastsync --justification --reason'
        optsParam='--domain|--ref|--lastsync|--justification|--reason'
        ;;

    del_sandbox)
        cmdOpts='--sandbox --justification --reason'
        optsParam='--sandbox|--justification|--reason'
        ;;

    manage)
        cmdOpts='--hostname --cluster --metacluster --list --domain --sandbox --force --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--list|--domain|--sandbox|--force|--justification|--reason'
        ;;

    cat)
        cmdOpts='--machine --hostname --cluster --metacluster --service --city --network_device --networkip --switch --personality --virtual_switch --generate --data --client --archetype --personality_stage --pre_feature --post_feature --param_tmpl --instance --default --server --resourcegroup --application --filesystem --hostlink --intervention --reboot_intervention --reboot_schedule --virtual_machine --service_address --share --network_environment'
        optsParam='--machine|--hostname|--cluster|--metacluster|--service|--city|--network_device|--networkip|--switch|--personality|--virtual_switch|--generate|--data|--client|--archetype|--personality_stage|--pre_feature|--post_feature|--param_tmpl|--instance|--default|--server|--resourcegroup|--application|--filesystem|--hostlink|--intervention|--reboot_intervention|--reboot_schedule|--virtual_machine|--service_address|--share|--network_environment'
        ;;

    sync)
        cmdOpts='--domain --justification --reason'
        optsParam='--domain|--justification|--reason'
        ;;

    add_chassis)
        cmdOpts='--chassis --rack --model --ip --interface --mac --label --vendor --serial --comments --justification --reason'
        optsParam='--chassis|--rack|--model|--ip|--interface|--mac|--label|--vendor|--serial|--comments|--justification|--reason'
        ;;

    show_chassis)
        cmdOpts='--chassis --all'
        optsParam='--chassis|--all'
        ;;

    del_chassis)
        cmdOpts='--chassis --clear_slots --justification --reason'
        optsParam='--chassis|--clear_slots|--justification|--reason'
        ;;

    update_chassis)
        cmdOpts='--chassis --model --rack --ip --vendor --serial --comments --justification --reason'
        optsParam='--chassis|--model|--rack|--ip|--vendor|--serial|--comments|--justification|--reason'
        ;;

    add_rack)
        cmdOpts='--rackid --row --column --building --room --bunker --fullname --comments --justification --reason'
        optsParam='--rackid|--row|--column|--building|--room|--bunker|--fullname|--comments|--justification|--reason'
        ;;

    update_rack)
        cmdOpts='--rack --row --column --fullname --comments --default_dns_domain --justification --reason --building --room --bunker'
        optsParam='--rack|--row|--column|--fullname|--comments|--default_dns_domain|--justification|--reason|--building|--room|--bunker'
        ;;

    show_rack)
        cmdOpts='--rack --all'
        optsParam='--rack|--all'
        ;;

    del_rack)
        cmdOpts='--rack --justification --reason'
        optsParam='--rack|--justification|--reason'
        ;;

    add_desk)
        cmdOpts='--desk --building --room --fullname --comments --justification --reason'
        optsParam='--desk|--building|--room|--fullname|--comments|--justification|--reason'
        ;;

    show_desk)
        cmdOpts='--desk --all'
        optsParam='--desk|--all'
        ;;

    del_desk)
        cmdOpts='--desk --justification --reason'
        optsParam='--desk|--justification|--reason'
        ;;

    add_building)
        cmdOpts='--building --address --city --fullname --comments --justification --reason'
        optsParam='--building|--address|--city|--fullname|--comments|--justification|--reason'
        ;;

    update_building)
        cmdOpts='--building --address --city --fullname --default_dns_domain --comments --justification --reason'
        optsParam='--building|--address|--city|--fullname|--default_dns_domain|--comments|--justification|--reason'
        ;;

    show_building)
        cmdOpts='--building --all'
        optsParam='--building|--all'
        ;;

    del_building)
        cmdOpts='--building --justification --reason'
        optsParam='--building|--justification|--reason'
        ;;

    add_campus)
        cmdOpts='--campus --country --fullname --comments --justification --reason'
        optsParam='--campus|--country|--fullname|--comments|--justification|--reason'
        ;;

    del_campus)
        cmdOpts='--campus --justification --reason'
        optsParam='--campus|--justification|--reason'
        ;;

    update_campus)
        cmdOpts='--campus --fullname --default_dns_domain --comments --justification --reason'
        optsParam='--campus|--fullname|--default_dns_domain|--comments|--justification|--reason'
        ;;

    add_room)
        cmdOpts='--room --building --fullname --comments --justification --reason'
        optsParam='--room|--building|--fullname|--comments|--justification|--reason'
        ;;

    show_room)
        cmdOpts='--room --all'
        optsParam='--room|--all'
        ;;

    del_room)
        cmdOpts='--room --justification --reason'
        optsParam='--room|--justification|--reason'
        ;;

    add_bunker)
        cmdOpts='--bunker --room --building --fullname --comments --justification --reason'
        optsParam='--bunker|--room|--building|--fullname|--comments|--justification|--reason'
        ;;

    show_bunker)
        cmdOpts='--bunker --all'
        optsParam='--bunker|--all'
        ;;

    del_bunker)
        cmdOpts='--bunker --justification --reason'
        optsParam='--bunker|--justification|--reason'
        ;;

    show_campus)
        cmdOpts='--campus --all'
        optsParam='--campus|--all'
        ;;

    add_city)
        cmdOpts='--city --timezone --country --campus --fullname --comments --justification --reason'
        optsParam='--city|--timezone|--country|--campus|--fullname|--comments|--justification|--reason'
        ;;

    update_city)
        cmdOpts='--city --timezone --fullname --campus --default_dns_domain --comments --justification --reason'
        optsParam='--city|--timezone|--fullname|--campus|--default_dns_domain|--comments|--justification|--reason'
        ;;

    show_city)
        cmdOpts='--city --all'
        optsParam='--city|--all'
        ;;

    del_city)
        cmdOpts='--city --justification --reason'
        optsParam='--city|--justification|--reason'
        ;;

    add_country)
        cmdOpts='--country --continent --fullname --comments --justification --reason'
        optsParam='--country|--continent|--fullname|--comments|--justification|--reason'
        ;;

    show_country)
        cmdOpts='--country --all'
        optsParam='--country|--all'
        ;;

    del_country)
        cmdOpts='--country --justification --reason'
        optsParam='--country|--justification|--reason'
        ;;

    add_continent)
        cmdOpts='--continent --hub --fullname --comments --justification --reason'
        optsParam='--continent|--hub|--fullname|--comments|--justification|--reason'
        ;;

    show_continent)
        cmdOpts='--continent --all'
        optsParam='--continent|--all'
        ;;

    del_continent)
        cmdOpts='--continent --justification --reason'
        optsParam='--continent|--justification|--reason'
        ;;

    add_organization)
        cmdOpts='--organization --fullname --comments --justification --reason'
        optsParam='--organization|--fullname|--comments|--justification|--reason'
        ;;

    del_organization)
        cmdOpts='--organization --justification --reason'
        optsParam='--organization|--justification|--reason'
        ;;

    show_organization)
        cmdOpts='--organization --all'
        optsParam='--organization|--all'
        ;;

    add_hub)
        cmdOpts='--hub --fullname --organization --comments --justification --reason'
        optsParam='--hub|--fullname|--organization|--comments|--justification|--reason'
        ;;

    show_hub)
        cmdOpts='--hub --all'
        optsParam='--hub|--all'
        ;;

    del_hub)
        cmdOpts='--hub --justification --reason'
        optsParam='--hub|--justification|--reason'
        ;;

    add_archetype)
        cmdOpts='--archetype --compilable --description --cluster_type --comments --justification --reason'
        optsParam='--archetype|--description|--cluster_type|--comments|--justification|--reason'
        ;;

    del_archetype)
        cmdOpts='--archetype --justification --reason'
        optsParam='--archetype|--justification|--reason'
        ;;

    add_personality)
        cmdOpts='--personality --archetype --copy_from --grn --eon_id --copy_stage --host_environment --cluster_required --comments --config_override --staged --justification --reason'
        optsParam='--personality|--archetype|--copy_from|--grn|--eon_id|--copy_stage|--host_environment|--cluster_required|--comments|--config_override|--staged|--justification|--reason'
        ;;

    add_vendor)
        cmdOpts='--vendor --comments --justification --reason'
        optsParam='--vendor|--comments|--justification|--reason'
        ;;

    del_vendor)
        cmdOpts='--vendor --justification --reason'
        optsParam='--vendor|--justification|--reason'
        ;;

    show_vendor)
        cmdOpts='--vendor --all'
        optsParam='--vendor|--all'
        ;;

    update_vendor)
        cmdOpts='--vendor --comments --justification --reason'
        optsParam='--vendor|--comments|--justification|--reason'
        ;;

    add_os)
        cmdOpts='--osname --osversion --archetype --comments --copy_version --justification --reason'
        optsParam='--osname|--osversion|--archetype|--comments|--copy_version|--justification|--reason'
        ;;

    update_os)
        cmdOpts='--osname --osversion --archetype --comments --justification --reason'
        optsParam='--osname|--osversion|--archetype|--comments|--justification|--reason'
        ;;

    del_os)
        cmdOpts='--osname --osversion --archetype --justification --reason'
        optsParam='--osname|--osversion|--archetype|--justification|--reason'
        ;;

    show_os)
        cmdOpts='--osname --osversion --archetype --all'
        optsParam='--osname|--osversion|--archetype|--all'
        ;;

    del_personality)
        cmdOpts='--personality --archetype --personality_stage --justification --reason'
        optsParam='--personality|--archetype|--personality_stage|--justification|--reason'
        ;;

    update_personality)
        cmdOpts='--personality --archetype --personality_stage --vmhost_capacity_function --cluster_required --config_override --host_environment --leave_existing --staged --justification --reason --comments --grn --eon_id'
        optsParam='--personality|--archetype|--personality_stage|--vmhost_capacity_function|--host_environment|--leave_existing|--justification|--reason|--comments|--grn|--eon_id'
        ;;

    show_personality)
        cmdOpts='--personality --archetype --all --personality_stage'
        optsParam='--personality|--archetype|--all|--personality_stage'
        ;;

    search_personality)
        cmdOpts='--personality --personality_stage --archetype --grn --eon_id --config_override --host_environment --required_service --fullinfo'
        optsParam='--personality|--personality_stage|--archetype|--grn|--eon_id|--host_environment|--required_service|--fullinfo'
        ;;

    make)
        cmdOpts='--hostname --osname --osversion --archetype --personality --personality_stage --buildstatus --keepbindings --cleargrn --comments --justification --reason --grn --eon_id'
        optsParam='--hostname|--osname|--osversion|--archetype|--personality|--personality_stage|--buildstatus|--cleargrn|--comments|--justification|--reason|--grn|--eon_id'
        ;;

    make_cluster)
        cmdOpts='--cluster --metacluster --keepbindings'
        optsParam='--cluster|--metacluster'
        ;;

    make_metacluster)
        cmdOpts='--metacluster --keepbindings --justification --reason'
        optsParam='--metacluster|--justification|--reason'
        ;;

    add_machine)
        cmdOpts='--machine --prefix --model --chassis --slot --rack --desk --cluster --metacluster --vmhost --vendor --recipe --cpuname --cpuvendor --cpucount --memory --serial --uuid --comments --uri --justification --reason'
        optsParam='--machine|--prefix|--model|--chassis|--slot|--rack|--desk|--cluster|--metacluster|--vmhost|--vendor|--recipe|--cpuname|--cpuvendor|--cpucount|--memory|--serial|--uuid|--comments|--uri|--justification|--reason'
        ;;

    update_machine)
        cmdOpts='--machine --hostname --chassis --slot --rack --desk --cluster --metacluster --vmhost --model --vendor --cpuname --cpuvendor --cpucount --memory --serial --uuid --clear_uuid --clearchassis --multislot --allow_metacluster_change --autoip --ip --uri --remap_disk --comments --justification --reason'
        optsParam='--machine|--hostname|--chassis|--slot|--rack|--desk|--cluster|--metacluster|--vmhost|--model|--vendor|--cpuname|--cpuvendor|--cpucount|--memory|--serial|--uuid|--clear_uuid|--autoip|--ip|--uri|--remap_disk|--comments|--justification|--reason'
        ;;

    show_machine)
        cmdOpts='--machine --all'
        optsParam='--machine|--all'
        ;;

    del_machine)
        cmdOpts='--machine --justification --reason'
        optsParam='--machine|--justification|--reason'
        ;;

    add_network_device)
        cmdOpts='--network_device --model --type --interface --iftype --ip --rack --building --archetype --personality --personality_stage --osname --osversion --buildstatus --mac --label --vendor --serial --comments --justification --reason --domain --sandbox --grn --eon_id'
        optsParam='--network_device|--model|--type|--interface|--iftype|--ip|--rack|--building|--archetype|--personality|--personality_stage|--osname|--osversion|--buildstatus|--mac|--label|--vendor|--serial|--comments|--justification|--reason|--domain|--sandbox|--grn|--eon_id'
        ;;

    add_switch)
        cmdOpts='--switch --model --type --ip --interface --rack --building --iftype --mac --label --vendor --serial --comments --justification --reason'
        optsParam='--switch|--model|--type|--ip|--interface|--rack|--building|--iftype|--mac|--label|--vendor|--serial|--comments|--justification|--reason'
        ;;

    update_network_device)
        cmdOpts='--network_device --rack --building --model --type --ip --vendor --serial --comments --rename_to --discovered_macs --clear --discover --justification --reason'
        optsParam='--network_device|--rack|--building|--model|--type|--ip|--vendor|--serial|--comments|--rename_to|--discovered_macs|--clear|--discover|--justification|--reason'
        ;;

    update_switch)
        cmdOpts='--switch --rack --building --model --type --ip --vendor --serial --comments --rename_to --discovered_macs --clear --discover --justification --reason'
        optsParam='--switch|--rack|--building|--model|--type|--ip|--vendor|--serial|--comments|--rename_to|--discovered_macs|--clear|--discover|--justification|--reason'
        ;;

    show_network_device)
        cmdOpts='--network_device --all --discover'
        optsParam='--network_device|--all|--discover'
        ;;

    show_switch)
        cmdOpts='--switch --all --discover'
        optsParam='--switch|--all|--discover'
        ;;

    search_network_device)
        cmdOpts='--network_device --ip --networkip --network_environment --vlan --type --model --vendor --organization --hub --continent --country --city --building --room --bunker --rack --desk --serial --exact_location --fullinfo --all'
        optsParam='--network_device|--ip|--networkip|--network_environment|--vlan|--type|--model|--vendor|--organization|--hub|--continent|--country|--city|--building|--room|--bunker|--rack|--desk|--serial|--exact_location|--fullinfo|--all'
        ;;

    search_switch)
        cmdOpts='--switch --ip --vlan --type --model --vendor --organization --hub --continent --country --city --building --room --bunker --rack --desk --serial --exact_location --fullinfo --all'
        optsParam='--switch|--ip|--vlan|--type|--model|--vendor|--organization|--hub|--continent|--country|--city|--building|--room|--bunker|--rack|--desk|--serial|--exact_location|--fullinfo|--all'
        ;;

    del_network_device)
        cmdOpts='--network_device --justification --reason'
        optsParam='--network_device|--justification|--reason'
        ;;

    del_switch)
        cmdOpts='--switch --justification --reason'
        optsParam='--switch|--justification|--reason'
        ;;

    poll_network_device)
        cmdOpts='--network_device --rack --type --clear --vlan --justification --reason'
        optsParam='--network_device|--rack|--type|--justification|--reason'
        ;;

    poll_switch)
        cmdOpts='--switch --rack --type --clear --vlan --justification --reason'
        optsParam='--switch|--rack|--type|--justification|--reason'
        ;;

    show_principal)
        cmdOpts='--principal --all'
        optsParam='--principal|--all'
        ;;

    search_principal)
        cmdOpts='--role --realm --fullinfo'
        optsParam='--role|--realm|--fullinfo'
        ;;

    show_realm)
        cmdOpts='--realm --all'
        optsParam='--realm|--all'
        ;;

    update_realm)
        cmdOpts='--realm --trusted --comments --justification --reason'
        optsParam='--realm|--comments|--justification|--reason'
        ;;

    permission)
        cmdOpts='--principal --role --createuser --createrealm --comments --justification --reason'
        optsParam='--principal|--role|--comments|--justification|--reason'
        ;;

    add_dns_environment)
        cmdOpts='--dns_environment --comments --justification --reason'
        optsParam='--dns_environment|--comments|--justification|--reason'
        ;;

    show_dns_environment)
        cmdOpts='--dns_environment --all'
        optsParam='--dns_environment|--all'
        ;;

    del_dns_environment)
        cmdOpts='--dns_environment --justification --reason'
        optsParam='--dns_environment|--justification|--reason'
        ;;

    update_dns_environment)
        cmdOpts='--dns_environment --comments --justification --reason'
        optsParam='--dns_environment|--comments|--justification|--reason'
        ;;

    add_dns_domain)
        cmdOpts='--dns_domain --comments --restricted --justification --reason'
        optsParam='--dns_domain|--comments|--restricted|--justification|--reason'
        ;;

    show_dns_domain)
        cmdOpts='--dns_domain --all'
        optsParam='--dns_domain|--all'
        ;;

    del_dns_domain)
        cmdOpts='--dns_domain --justification --reason'
        optsParam='--dns_domain|--justification|--reason'
        ;;

    update_dns_domain)
        cmdOpts='--dns_domain --comments --restricted --justification --reason'
        optsParam='--dns_domain|--comments|--justification|--reason'
        ;;

    dump_dns)
        cmdOpts='--dns_domain --dns_environment'
        optsParam='--dns_domain|--dns_environment'
        ;;

    add_metacluster)
        cmdOpts='--metacluster --prefix --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --domain --sandbox --archetype --personality --personality_stage --max_members --buildstatus --comments --justification --reason'
        optsParam='--metacluster|--prefix|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--domain|--sandbox|--archetype|--personality|--personality_stage|--max_members|--buildstatus|--comments|--justification|--reason'
        ;;

    show_metacluster)
        cmdOpts='--metacluster --all'
        optsParam='--metacluster|--all'
        ;;

    update_metacluster)
        cmdOpts='--metacluster --personality --personality_stage --max_members --comments --fix_location --virtual_switch --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --justification --reason'
        optsParam='--metacluster|--personality|--personality_stage|--max_members|--comments|--fix_location|--virtual_switch|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--justification|--reason'
        ;;

    del_metacluster)
        cmdOpts='--metacluster --justification --reason'
        optsParam='--metacluster|--justification|--reason'
        ;;

    show_cluster)
        cmdOpts='--cluster --all'
        optsParam='--cluster|--all'
        ;;

    add_esx_cluster)
        cmdOpts='--cluster --metacluster --down_hosts_threshold --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --archetype --personality --domain --sandbox --personality_stage --maint_threshold --max_members --vm_to_host_ratio --switch --buildstatus --comments --justification --reason'
        optsParam='--cluster|--metacluster|--down_hosts_threshold|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--archetype|--personality|--domain|--sandbox|--personality_stage|--maint_threshold|--max_members|--vm_to_host_ratio|--switch|--buildstatus|--comments|--justification|--reason'
        ;;

    show_esx_cluster)
        cmdOpts='--cluster'
        optsParam='--cluster'
        ;;

    update_esx_cluster)
        cmdOpts='--cluster --personality --personality_stage --down_hosts_threshold --maint_threshold --max_members --switch --fix_location --comments --virtual_switch --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --justification --reason'
        optsParam='--cluster|--personality|--personality_stage|--down_hosts_threshold|--maint_threshold|--max_members|--switch|--fix_location|--comments|--virtual_switch|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--justification|--reason'
        ;;

    del_esx_cluster)
        cmdOpts='--cluster --justification --reason'
        optsParam='--cluster|--justification|--reason'
        ;;

    add_cluster)
        cmdOpts='--cluster --prefix --archetype --personality --down_hosts_threshold --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --domain --sandbox --metacluster --personality_stage --maint_threshold --max_members --vm_to_host_ratio --switch --buildstatus --comments --justification --reason'
        optsParam='--cluster|--prefix|--archetype|--personality|--down_hosts_threshold|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--domain|--sandbox|--metacluster|--personality_stage|--maint_threshold|--max_members|--vm_to_host_ratio|--switch|--buildstatus|--comments|--justification|--reason'
        ;;

    update_cluster)
        cmdOpts='--cluster --personality --personality_stage --down_hosts_threshold --maint_threshold --max_members --switch --fix_location --comments --virtual_switch --metacluster --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --group_with --clear_group --justification --reason'
        optsParam='--cluster|--personality|--personality_stage|--down_hosts_threshold|--maint_threshold|--max_members|--switch|--fix_location|--comments|--virtual_switch|--metacluster|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--group_with|--clear_group|--justification|--reason'
        ;;

    del_cluster)
        cmdOpts='--cluster --justification --reason'
        optsParam='--cluster|--justification|--reason'
        ;;

    add_allowed_personality)
        cmdOpts='--cluster --metacluster --archetype --personality --justification --reason'
        optsParam='--cluster|--metacluster|--archetype|--personality|--justification|--reason'
        ;;

    del_allowed_personality)
        cmdOpts='--cluster --metacluster --archetype --personality --justification --reason'
        optsParam='--cluster|--metacluster|--archetype|--personality|--justification|--reason'
        ;;

    rebind_metacluster)
        cmdOpts='--metacluster --cluster --justification --reason'
        optsParam='--metacluster|--cluster|--justification|--reason'
        ;;

    bind_cluster)
        cmdOpts='--cluster --service --instance --justification --reason'
        optsParam='--cluster|--service|--instance|--justification|--reason'
        ;;

    rebind_cluster)
        cmdOpts='--cluster --service --instance --justification --reason'
        optsParam='--cluster|--service|--instance|--justification|--reason'
        ;;

    unbind_cluster)
        cmdOpts='--cluster --service --justification --reason'
        optsParam='--cluster|--service|--justification|--reason'
        ;;

    cluster)
        cmdOpts='--hostname --cluster --personality --personality_stage --justification --reason'
        optsParam='--hostname|--cluster|--personality|--personality_stage|--justification|--reason'
        ;;

    uncluster)
        cmdOpts='--hostname --cluster --personality --personality_stage --justification --reason'
        optsParam='--hostname|--cluster|--personality|--personality_stage|--justification|--reason'
        ;;

    add_dynamic_range)
        cmdOpts='--startip --endip --fillnetwork --dns_domain --prefix --justification --reason'
        optsParam='--startip|--endip|--fillnetwork|--dns_domain|--prefix|--justification|--reason'
        ;;

    del_dynamic_range)
        cmdOpts='--startip --endip --clearnetwork --justification --reason'
        optsParam='--startip|--endip|--clearnetwork|--justification|--reason'
        ;;

    show_dynamic_range)
        cmdOpts='--ip --fqdn --dns_environment'
        optsParam='--ip|--fqdn|--dns_environment'
        ;;

    add_address)
        cmdOpts='--fqdn --ip --ipfromip --ipfromsystem --dns_environment --network_environment --ipalgorithm --ttl --comments --reverse_ptr --grn --eon_id --justification --reason'
        optsParam='--fqdn|--ip|--ipfromip|--ipfromsystem|--dns_environment|--network_environment|--ipalgorithm|--ttl|--comments|--reverse_ptr|--grn|--eon_id|--justification|--reason'
        ;;

    del_address)
        cmdOpts='--fqdn --ip --dns_environment --network_environment --justification --reason'
        optsParam='--fqdn|--ip|--dns_environment|--network_environment|--justification|--reason'
        ;;

    show_address)
        cmdOpts='--fqdn --dns_environment --network_environment'
        optsParam='--fqdn|--dns_environment|--network_environment'
        ;;

    update_address)
        cmdOpts='--fqdn --ip --ttl --clear_ttl --comments --reverse_ptr --dns_environment --network_environment --grn --eon_id --clear_grn --justification --reason'
        optsParam='--fqdn|--ip|--ttl|--clear_ttl|--comments|--reverse_ptr|--dns_environment|--network_environment|--grn|--eon_id|--clear_grn|--justification|--reason'
        ;;

    show_ns_record)
        cmdOpts='--dns_domain --fqdn --all'
        optsParam='--dns_domain|--fqdn|--all'
        ;;

    add_ns_record)
        cmdOpts='--fqdn --dns_domain --comments --justification --reason'
        optsParam='--fqdn|--dns_domain|--comments|--justification|--reason'
        ;;

    del_ns_record)
        cmdOpts='--fqdn --dns_domain --justification --reason'
        optsParam='--fqdn|--dns_domain|--justification|--reason'
        ;;

    add_alias)
        cmdOpts='--fqdn --target --ttl --comments --dns_environment --target_environment --grn --eon_id --justification --reason'
        optsParam='--fqdn|--target|--ttl|--comments|--dns_environment|--target_environment|--grn|--eon_id|--justification|--reason'
        ;;

    del_alias)
        cmdOpts='--fqdn --dns_environment --justification --reason'
        optsParam='--fqdn|--dns_environment|--justification|--reason'
        ;;

    add_address_alias)
        cmdOpts='--fqdn --target --ttl --comments --dns_environment --target_environment --grn --eon_id --justification --reason'
        optsParam='--fqdn|--target|--ttl|--comments|--dns_environment|--target_environment|--grn|--eon_id|--justification|--reason'
        ;;

    del_address_alias)
        cmdOpts='--fqdn --target --dns_environment --target_environment --justification --reason'
        optsParam='--fqdn|--target|--dns_environment|--target_environment|--justification|--reason'
        ;;

    update_address_alias)
        cmdOpts='--fqdn --target --ttl --clear_ttl --comments --dns_environment --target_environment --grn --eon_id --clear_grn --justification --reason'
        optsParam='--fqdn|--target|--ttl|--clear_ttl|--comments|--dns_environment|--target_environment|--grn|--eon_id|--clear_grn|--justification|--reason'
        ;;

    show_alias)
        cmdOpts='--fqdn --dns_environment'
        optsParam='--fqdn|--dns_environment'
        ;;

    update_alias)
        cmdOpts='--fqdn --target --ttl --clear_ttl --comments --dns_environment --target_environment --grn --eon_id --clear_grn --justification --reason'
        optsParam='--fqdn|--target|--ttl|--clear_ttl|--comments|--dns_environment|--target_environment|--grn|--eon_id|--clear_grn|--justification|--reason'
        ;;

    add_srv_record)
        cmdOpts='--service --protocol --dns_domain --priority --weight --target --port --ttl --comments --dns_environment --grn --eon_id --justification --reason'
        optsParam='--service|--protocol|--dns_domain|--priority|--weight|--target|--port|--ttl|--comments|--dns_environment|--grn|--eon_id|--justification|--reason'
        ;;

    del_srv_record)
        cmdOpts='--service --protocol --dns_domain --dns_environment --target --justification --reason'
        optsParam='--service|--protocol|--dns_domain|--dns_environment|--target|--justification|--reason'
        ;;

    show_srv_record)
        cmdOpts='--service --protocol --dns_domain --dns_environment'
        optsParam='--service|--protocol|--dns_domain|--dns_environment'
        ;;

    update_srv_record)
        cmdOpts='--service --protocol --dns_domain --target --priority --weight --port --ttl --clear_ttl --comments --dns_environment --grn --eon_id --clear_grn --justification --reason'
        optsParam='--service|--protocol|--dns_domain|--target|--priority|--weight|--port|--ttl|--clear_ttl|--comments|--dns_environment|--grn|--eon_id|--clear_grn|--justification|--reason'
        ;;

    search_dns)
        cmdOpts='--fqdn --shortname --record_type --ip --network --target --target_domain --dns_domain --primary_name --used --reverse_override --reverse_ptr --dns_environment --network_environment --target_environment --fullinfo'
        optsParam='--fqdn|--shortname|--record_type|--ip|--network|--target|--target_domain|--dns_domain|--reverse_ptr|--dns_environment|--network_environment|--target_environment|--fullinfo'
        ;;

    search_rack)
        cmdOpts='--organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --all --row --column --fullinfo'
        optsParam='--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--all|--row|--column|--fullinfo'
        ;;

    map_dns_domain)
        cmdOpts='--dns_domain --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --position --comments --justification --reason'
        optsParam='--dns_domain|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--position|--comments|--justification|--reason'
        ;;

    unmap_dns_domain)
        cmdOpts='--dns_domain --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --justification --reason'
        optsParam='--dns_domain|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--justification|--reason'
        ;;

    search_dns_domain_map)
        cmdOpts='--dns_domain --organization --hub --continent --country --campus --city --building --room --bunker --rack --desk --include_parents'
        optsParam='--dns_domain|--organization|--hub|--continent|--country|--campus|--city|--building|--room|--bunker|--rack|--desk|--include_parents'
        ;;

    search_audit)
        cmdOpts='--keyword --argument --username --command --before --after --return_code --limit --reverse_order'
        optsParam='--keyword|--argument|--username|--command|--before|--after|--return_code|--limit|--reverse_order'
        ;;

    show_reboot_schedule)
        cmdOpts='--cluster --hostname --all'
        optsParam='--cluster|--hostname|--all'
        ;;

    add_reboot_schedule)
        cmdOpts='--hostname --cluster --week --day --time --comments --justification --reason'
        optsParam='--hostname|--cluster|--week|--day|--time|--comments|--justification|--reason'
        ;;

    del_reboot_schedule)
        cmdOpts='--hostname --cluster --justification --reason'
        optsParam='--hostname|--cluster|--justification|--reason'
        ;;

    show_reboot_intervention)
        cmdOpts='--cluster --hostname --all'
        optsParam='--cluster|--hostname|--all'
        ;;

    add_reboot_intervention)
        cmdOpts='--hostname --cluster --expiry --justification --reason --comments --start_time'
        optsParam='--hostname|--cluster|--expiry|--justification|--reason|--comments|--start_time'
        ;;

    del_reboot_intervention)
        cmdOpts='--hostname --cluster --justification --reason'
        optsParam='--hostname|--cluster|--justification|--reason'
        ;;

    add_filesystem)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --filesystem --type --mountpoint --blockdevice --bootmount --dumpfreq --fsckpass --options --comments --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--filesystem|--type|--mountpoint|--blockdevice|--dumpfreq|--fsckpass|--options|--comments|--justification|--reason'
        ;;

    del_filesystem)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --filesystem --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--filesystem|--justification|--reason'
        ;;

    show_filesystem)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --filesystem --all'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--filesystem|--all'
        ;;

    add_share)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --share --comments --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--share|--comments|--justification|--reason'
        ;;

    update_share)
        cmdOpts='--share --comments --latency_threshold --justification --reason'
        optsParam='--share|--comments|--latency_threshold|--justification|--reason'
        ;;

    del_share)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --share --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--share|--justification|--reason'
        ;;

    show_share)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --share --all'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--share|--all'
        ;;

    add_application)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --application --eonid --eon_id --grn --comments --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--application|--eonid|--eon_id|--grn|--comments|--justification|--reason'
        ;;

    del_application)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --application --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--application|--justification|--reason'
        ;;

    show_application)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --application --all'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--application|--all'
        ;;

    add_hostlink)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --hostlink --target --owner --group --comments --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--hostlink|--target|--owner|--group|--comments|--justification|--reason'
        ;;

    del_hostlink)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --hostlink --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--hostlink|--justification|--reason'
        ;;

    show_hostlink)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --hostlink --all'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--hostlink|--all'
        ;;

    add_intervention)
        cmdOpts='--hostname --cluster --metacluster --expiry --intervention --justification --reason --comments --allowusers --allowgroups --disabled_actions --start_time'
        optsParam='--hostname|--cluster|--metacluster|--expiry|--intervention|--justification|--reason|--comments|--allowusers|--allowgroups|--disabled_actions|--start_time'
        ;;

    show_intervention)
        cmdOpts='--hostname --cluster --metacluster --intervention --all'
        optsParam='--hostname|--cluster|--metacluster|--intervention|--all'
        ;;

    del_intervention)
        cmdOpts='--hostname --cluster --metacluster --intervention --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--intervention|--justification|--reason'
        ;;

    add_service_address)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --service_address --name --interfaces --ip --network_environment --comments --map_to_primary --shared --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--service_address|--name|--interfaces|--ip|--network_environment|--comments|--shared|--justification|--reason'
        ;;

    del_service_address)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --name --keep_dns --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--name|--keep_dns|--justification|--reason'
        ;;

    show_service_address)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --name --all'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--name|--all'
        ;;

    update_service_address)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --name --interfaces --ip --network_environment --comments --map_to_primary --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--name|--interfaces|--ip|--network_environment|--comments|--justification|--reason'
        ;;

    add_grn)
        cmdOpts='--grn --eon_id --disabled --justification --reason'
        optsParam='--grn|--eon_id|--disabled|--justification|--reason'
        ;;

    del_grn)
        cmdOpts='--grn --eon_id --justification --reason'
        optsParam='--grn|--eon_id|--justification|--reason'
        ;;

    update_grn)
        cmdOpts='--grn --eon_id --disabled --rename_to --justification --reason'
        optsParam='--grn|--eon_id|--rename_to|--justification|--reason'
        ;;

    show_grn)
        cmdOpts='--grn --eon_id --all'
        optsParam='--grn|--eon_id|--all'
        ;;

    refresh_grns)
        cmdOpts='--justification --reason'
        optsParam='--justification|--reason'
        ;;

    map_grn)
        cmdOpts='--target --grn --eon_id --hostname --list --personality --membersof --archetype --personality_stage --justification --reason'
        optsParam='--target|--grn|--eon_id|--hostname|--list|--personality|--membersof|--archetype|--personality_stage|--justification|--reason'
        ;;

    unmap_grn)
        cmdOpts='--target --grn --eon_id --clearall --hostname --list --membersof --personality --archetype --personality_stage --justification --reason'
        optsParam='--target|--grn|--eon_id|--clearall|--hostname|--list|--membersof|--personality|--archetype|--personality_stage|--justification|--reason'
        ;;

    add_resourcegroup)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --required_type --comments --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--required_type|--comments|--justification|--reason'
        ;;

    del_resourcegroup)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--justification|--reason'
        ;;

    show_resourcegroup)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --all'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--all'
        ;;

    update_resourcegroup)
        cmdOpts='--hostname --cluster --metacluster --resourcegroup --required_type --comments --justification --reason'
        optsParam='--hostname|--cluster|--metacluster|--resourcegroup|--required_type|--comments|--justification|--reason'
        ;;

    add_feature)
        cmdOpts='--feature --type --activation --deactivation --grn --eon_id --post_personality --comments --visibility --justification --reason'
        optsParam='--feature|--type|--activation|--deactivation|--grn|--eon_id|--post_personality|--comments|--visibility|--justification|--reason'
        ;;

    update_feature)
        cmdOpts='--feature --type --comments --visibility --activation --deactivation --grn --eon_id --justification --reason'
        optsParam='--feature|--type|--comments|--visibility|--activation|--deactivation|--grn|--eon_id|--justification|--reason'
        ;;

    del_feature)
        cmdOpts='--feature --type --justification --reason'
        optsParam='--feature|--type|--justification|--reason'
        ;;

    show_feature)
        cmdOpts='--feature --type --all'
        optsParam='--feature|--type|--all'
        ;;

    bind_feature)
        cmdOpts='--feature --personality --archetype --model --interface --personality_stage --vendor --justification --reason'
        optsParam='--feature|--personality|--archetype|--model|--interface|--personality_stage|--vendor|--justification|--reason'
        ;;

    unbind_feature)
        cmdOpts='--feature --personality --archetype --model --interface --personality_stage --vendor --justification --reason'
        optsParam='--feature|--personality|--archetype|--model|--interface|--personality_stage|--vendor|--justification|--reason'
        ;;

    show_parameter)
        cmdOpts='--personality --archetype --personality_stage'
        optsParam='--personality|--archetype|--personality_stage'
        ;;

    validate_parameter)
        cmdOpts='--personality --archetype --personality_stage'
        optsParam='--personality|--archetype|--personality_stage'
        ;;

    add_parameter)
        cmdOpts='--personality --path --feature --type --archetype --personality_stage --value --justification --reason'
        optsParam='--personality|--path|--feature|--type|--archetype|--personality_stage|--value|--justification|--reason'
        ;;

    update_parameter)
        cmdOpts='--personality --path --value --feature --type --archetype --personality_stage --justification --reason'
        optsParam='--personality|--path|--value|--feature|--type|--archetype|--personality_stage|--justification|--reason'
        ;;

    del_parameter)
        cmdOpts='--personality --path --feature --type --archetype --personality_stage --justification --reason'
        optsParam='--personality|--path|--feature|--type|--archetype|--personality_stage|--justification|--reason'
        ;;

    show_diff)
        cmdOpts='--archetype --personality --other --other_stage --personality_stage --other_archetype'
        optsParam='--archetype|--personality|--other|--other_stage|--personality_stage|--other_archetype'
        ;;

    add_parameter_definition)
        cmdOpts='--archetype --template --feature --type --path --value_type --required --default --description --schema --activation --justification --reason'
        optsParam='--archetype|--template|--feature|--type|--path|--value_type|--required|--default|--description|--schema|--activation|--justification|--reason'
        ;;

    del_parameter_definition)
        cmdOpts='--archetype --feature --type --path --justification --reason'
        optsParam='--archetype|--feature|--type|--path|--justification|--reason'
        ;;

    update_parameter_definition)
        cmdOpts='--archetype --feature --type --path --required --default --clear_default --description --schema --clear_schema --activation --justification --reason'
        optsParam='--archetype|--feature|--type|--path|--default|--clear_default|--description|--schema|--clear_schema|--activation|--justification|--reason'
        ;;

    show_parameter_definition)
        cmdOpts='--archetype --feature --type --path'
        optsParam='--archetype|--feature|--type|--path'
        ;;

    search_parameter_definition)
        cmdOpts='--archetype --feature --type --template'
        optsParam='--archetype|--feature|--type|--template'
        ;;

    search_parameter)
        cmdOpts='--archetype --path'
        optsParam='--archetype|--path'
        ;;

    add_vlan)
        cmdOpts='--vlan --name --vlan_type --justification --reason'
        optsParam='--vlan|--name|--vlan_type|--justification|--reason'
        ;;

    del_vlan)
        cmdOpts='--vlan --justification --reason'
        optsParam='--vlan|--justification|--reason'
        ;;

    show_vlan)
        cmdOpts='--vlan --all'
        optsParam='--vlan|--all'
        ;;

    show_bunker_violations)
        cmdOpts='--management_interfaces'
        optsParam=''
        ;;

    grant_root_access)
        cmdOpts='--username --netgroup --personality --justification --archetype --reason'
        optsParam='--username|--netgroup|--personality|--justification|--archetype|--reason'
        ;;

    revoke_root_access)
        cmdOpts='--username --netgroup --personality --justification --archetype --reason'
        optsParam='--username|--netgroup|--personality|--justification|--archetype|--reason'
        ;;

    add_netgroup_whitelist)
        cmdOpts='--netgroup --justification --reason'
        optsParam='--netgroup|--justification|--reason'
        ;;

    del_netgroup_whitelist)
        cmdOpts='--netgroup --justification --reason'
        optsParam='--netgroup|--justification|--reason'
        ;;

    show_netgroup_whitelist)
        cmdOpts='--netgroup --all'
        optsParam='--netgroup|--all'
        ;;

    add_user)
        cmdOpts='--username --uid --autouid --gid --full_name --home_directory --justification --reason'
        optsParam='--username|--uid|--autouid|--gid|--full_name|--home_directory|--justification|--reason'
        ;;

    del_user)
        cmdOpts='--username --justification --reason'
        optsParam='--username|--justification|--reason'
        ;;

    update_user)
        cmdOpts='--username --uid --gid --full_name --home_directory --justification --reason'
        optsParam='--username|--uid|--gid|--full_name|--home_directory|--justification|--reason'
        ;;

    show_user)
        cmdOpts='--username --all'
        optsParam='--username|--all'
        ;;

    refresh_user)
        cmdOpts='--incremental --ignore_delete_limit --justification --reason'
        optsParam='--justification|--reason'
        ;;

    add_virtual_switch)
        cmdOpts='--virtual_switch --comments --justification --reason'
        optsParam='--virtual_switch|--comments|--justification|--reason'
        ;;

    del_virtual_switch)
        cmdOpts='--virtual_switch --justification --reason'
        optsParam='--virtual_switch|--justification|--reason'
        ;;

    show_virtual_switch)
        cmdOpts='--virtual_switch --all'
        optsParam='--virtual_switch|--all'
        ;;

    update_virtual_switch)
        cmdOpts='--virtual_switch --comments --justification --reason'
        optsParam='--virtual_switch|--comments|--justification|--reason'
        ;;

    bind_port_group)
        cmdOpts='--virtual_switch --networkip --tag --type --justification --reason'
        optsParam='--virtual_switch|--networkip|--tag|--type|--justification|--reason'
        ;;

    unbind_port_group)
        cmdOpts='--virtual_switch --networkip --tag --justification --reason'
        optsParam='--virtual_switch|--networkip|--tag|--justification|--reason'
        ;;

    promote)
        cmdOpts='--personality --archetype --justification --reason'
        optsParam='--personality|--archetype|--justification|--reason'
        ;;
    esac

    # append global options
    # cmdOpts="$cmdOpts $globalOpts"
    # optsParam="$optsParam|$globalParams"

    # take out options already given
    for (( i = 2; i<= $COMP_CWORD-1; ++i)); do
        opt=${COMP_WORDS[$i]}

        case $opt in
            --*) optBase=${opt/=*/} ;;
            -*)  optBase=${opt:0:2} ;;
        esac

        cmdOpts=" $cmdOpts "
        cmdOpts=${cmdOpts/ ${optBase} / }

        # skip next option if this one requires a parameter
        if [[ "$opt" =~ "$optsParam" ]]; then
            ((++i))
        fi
    done

    COMPREPLY=( $( compgen -W "$cmdOpts" -- $cur ) )

    return 0
}

complete -F _aq -o default aq
